@@ -1,14 +1,14 @@
 #' Data Cleaning
 #'
 #'
-#'The \code{data_cleansing} function is a simpler wrapper for data cleaning functions, such as 
+#'The \code{data_cleansing} function is a simpler wrapper for data cleaning functions, such as
 #' delete variables that values are all NAs;
 #' checking dat and target format.
 #' delete low variance variables
-#' replace null or NULL or blank with NA; 
+#' replace null or NULL or blank with NA;
 #' encode variables which NAs &  miss value rate is more than 95% as 1,0 ;
 #' encode variables which unique value  rate is  more than 95% as 1,0;
-#'merge categories of character variables that  is more than 8; 
+#' merge categories of character variables that  is more than 10;
 #' transfer time variables to dateformation;
 #' remove duplicated observations;
 #' process outliers;
@@ -24,7 +24,12 @@
 #' @param outlier_proc  Logical, process outliers or not. Default is TRUE.
 #' @param missing_proc  Logical, process nas or not. Default is TRUE.
 #' @param low_var  Logical, delete low variance variables or not. Default is TRUE.
+#' @param lvp  The maximum percent of unique values (including NAs).
+#' @param nr  The maximum percent of NAs.
 #' @param one_hot  Logical. If TRUE, one-hot_encoding  of category variables. Default is FASLE.
+#' @param merge_cat merge categories of character variables that  is more than m.
+#' @param p The minimum percent of samples in a category to merge.
+#' @param m The minimum number of categories.
 #' @param parallel  Logical, parallel computing or not. Default is FALSE.
 #' @param note Logical. Outputs info. Default is TRUE.
 #' @param save_data  Logical, save the result or not. Default is FALSE.
@@ -59,14 +64,14 @@
 
 
 
-data_cleansing <- function(dat, target = NULL, x_list = NULL, obs_id = NULL, occur_time = NULL,
+data_cleansing <- function(dat, target = NULL, obs_id = NULL, occur_time = NULL,x_list = NULL,
                           pos_flag = NULL, miss_values = NULL, ex_cols = NULL,
-                          outlier_proc = TRUE, missing_proc = TRUE,
-                          low_var = TRUE, one_hot = FALSE,
+                          outlier_proc = TRUE, missing_proc = TRUE, merge_cat = TRUE, p = 0.001, m = 20,
+                          low_var = TRUE, lvp = 0.99,nr = 0.99, one_hot = FALSE,
                           parallel = FALSE, note = FALSE,
                           save_data = FALSE, file_name = NULL, dir_path = tempdir()) {
     #delete variables that values are all nas.
-    opt = options(scipen = 200, "warn" = -1, stringsAsFactors = FALSE, digits = 10) # suppress warnings
+    opt = options(scipen = 200, "warn" = -1, stringsAsFactors = FALSE, digits = 10)
     dat = checking_data(dat = dat, target = target, pos_flag = pos_flag,
                             occur_time = occur_time, note = note)
     if (save_data) {
@@ -74,15 +79,15 @@ data_cleansing <- function(dat, target = NULL, x_list = NULL, obs_id = NULL, occ
         if (!dir.exists(dir_path)) dir.create(dir_path)
         if (!is.character(file_name)) file_name = NULL
         }
-    if (note) cat("[NOTE] data cleansing.\n")
+    if (note)cat_line("-- Cleansing data", col = love_color("dark_green"))
 
     x_list = get_names(dat = dat,
-                              types = c('logical', 'factor', 'character', 'numeric',
+                              types = c('logical', 'factor', 'character', 'numeric','integer64',
                                         'integer', 'double', "Date", "POSIXlt", "POSIXct", "POSIXt"),
                               ex_cols = ex_cols, get_ex = FALSE)
     ex_x_cols = ex_x_cols2 = NULL
     ex_x_cols = get_names(dat = dat,
-                              types = c('logical', 'factor', 'character', 'numeric',
+                              types = c('logical', 'factor', 'character', 'numeric','integer64',
                                         'integer', 'double', "Date", "POSIXlt", "POSIXct", "POSIXt"),
                               ex_cols = ex_cols, get_ex = TRUE)
     if (length(dat) > 0) {
@@ -96,33 +101,39 @@ data_cleansing <- function(dat, target = NULL, x_list = NULL, obs_id = NULL, occ
         if (low_var) {
             #delecte low vaiance variables
             dat = dat[!colAllnas(dat)] %>%
-                      low_variance_filter(lvp = 0.98, note = note)
+                      low_variance_filter(lvp = lvp, note = note)
             x_list_2 = get_names(dat = dat,
-                              types = c('logical', 'factor', 'character', 'numeric',
+                              types = c('logical', 'factor', 'character', 'numeric','integer64',
                                         'integer', 'double', "Date", "POSIXlt", "POSIXct", "POSIXt"),
                               ex_cols = ex_cols, get_ex = FALSE)
             ex_x_cols2 = setdiff(x_list, x_list_2)
             ex_x_cols = unique(c(ex_x_cols, ex_x_cols2))
         }
         if (length(ex_x_cols) > 0) {
-            cat(paste("[NOTE] exclued variables : ", paste(ex_x_cols, collapse = " , "), ".\n",
-                      sep = "", collapse = "\n"))
+		
+		   cat_line("-- Following variables are excluded:", col = love_color("dark_green"))
+            cat_bullet(paste0(format(ex_x_cols)), col = "darkgrey")
+
         }
         #transfer time variables to date formation
-        dat = time_transfer(dat = dat, date_cols = NULL,
-        ex_cols = c(obs_id, target, occur_time, ex_cols),
-                        note = note) %>%
-          char_to_num(ex_cols = c(obs_id, target, occur_time, ex_cols), note = note) %>%
-        remove_duplicated(target = target, obs_id = obs_id, occur_time = occur_time, note = note) %>% entry_rate_na(nr = 0.95, note = note) %>%
-        #merge categories of character variables that  is more than 8
-        merge_category(ex_cols = c(obs_id, target, occur_time, ex_cols),
-                         p = 0.001, m = 20, note = note)
+        dat = time_transfer(dat = dat, date_cols = NULL, ex_cols = c(obs_id, target, ex_cols), note = note) %>%
+        char_to_num(ex_cols = c(obs_id, target, occur_time, ex_cols), note = note) %>%
+        remove_duplicated(obs_id = obs_id, occur_time = occur_time, note = note) %>%
+		entry_rate_na(nr = nr, note = note)
+        #merge categories of character variables that  is more than 20.
+		if(merge_cat){
+
+		dat = merge_category(dat = dat, ex_cols = c(obs_id, target, occur_time, ex_cols),
+                         p =p, m = m, note = note)
+
+		}
+
         char_x_list = get_names(dat = dat,
                                 types = c('factor', 'character'),
                                 ex_cols = c(obs_id, target, occur_time, ex_cols),
                                 get_ex = FALSE)
         num_x_list = get_names(dat = dat,
-                               types = c('numeric', 'integer', 'double'),
+                               types = c('numeric', 'integer', 'double','integer64'),
                                ex_cols = c(obs_id, target, occur_time, ex_cols),
                                get_ex = FALSE)
         date_x_list = get_names(dat = dat,
@@ -135,11 +146,7 @@ data_cleansing <- function(dat, target = NULL, x_list = NULL, obs_id = NULL, occ
                               ex_cols = c(obs_id, occur_time, target),
                               get_ex = TRUE)
         dat = dat[, c(flag_list, date_x_list, char_x_list, num_x_list)]
-       re_x_list = gsub("-|\\$|\\*|\\+|\\?|\\[|\\^|\\{|\\}|\\\\|\\(|\\)|\\|\\)|\\]", "", c(char_x_list, num_x_list))
-        if (one_hot & length(char_x_list) > 0) {
-            dat = one_hot_encoding(dat = dat, cat_vars = char_x_list,
-                                  na_act = FALSE, note = note)
-        }
+
         if (outlier_proc) {
             dat = process_outliers(dat = dat, target = target,
                                    x_list = num_x_list, ex_cols = c(obs_id, occur_time, target),
@@ -156,10 +163,16 @@ data_cleansing <- function(dat, target = NULL, x_list = NULL, obs_id = NULL, occ
                               default_miss = TRUE, parallel = parallel,
                               method = "median", note = note, save_data = save_data, file_name = file_name, dir_path = dir_path)
         }
-        re_x_list = gsub("-|\\$|\\*|\\+|\\?|\\[|\\^|\\{|\\}|\\\\|\\(|\\)|\\|\\)|\\]", "_", c(char_x_list, num_x_list))
+		
+		re_x_list = gsub("[^\u4e00-\u9fa5,^a-zA-Z,^0-9,^_]", "", c(char_x_list, num_x_list))
+
         dat = re_name(dat, oldname = c(char_x_list, num_x_list), newname = re_x_list)
+		if (one_hot & length(char_x_list) > 0) {
+            dat = one_hot_encoding(dat = dat, cat_vars = char_x_list, na_act = FALSE, note = note)
+        }
+
         if (save_data) {
-            save_dt(dat, dir_path = dir_path, file_name = ifelse(is.null(file_name), "data.cleansing", paste(file_name, "data.cleansing", sep = ".")), append = FALSE, note = note)
+            save_dt(dat, dir_path = dir_path, file_name = ifelse(is.null(file_name), "data_cleansing", paste(file_name, "data_cleansing", sep = ".")), append = FALSE, note = note)
         }
     }
     options(opt) # reset warnings
@@ -189,7 +202,9 @@ data_cleansing <- function(dat, target = NULL, x_list = NULL, obs_id = NULL, occ
 
 remove_duplicated <- function(dat = dat, obs_id = NULL, occur_time = NULL,
                               target = NULL , note = FALSE) {
-    if(note)cat("[NOTE]", "remove duplicated observations...\n")
+    if(note)cat_line("-- Removing duplicated observations", col = love_color("dark_green"))
+	
+
     y_1 = id_1 = apply_date_1 = NULL
     if (!is.null(obs_id)) {
         if (!is.null(target)) {
@@ -254,7 +269,7 @@ remove_duplicated <- function(dat = dat, obs_id = NULL, occur_time = NULL,
 #' @export
 #' @importFrom dplyr group_by mutate summarize  summarise n  count %>% filter mutate_if
 null_blank_na <- function(dat, miss_values = NULL, note = FALSE) {
-    if(note)cat("[NOTE]", "replace null or blank or miss_values, with NA\n")
+    if(note)cat_line("-- Replacing null or blank or miss_values with NA", col = love_color("dark_green"))
     dat %>%
       mutate_if(is.character, function(x) {
         ifelse(is.null(x) | x == "" | x == "null" | x == "NULL" | x == " " | x %in% miss_values, NA, x)}) %>%
@@ -279,8 +294,8 @@ null_blank_na <- function(dat, miss_values = NULL, note = FALSE) {
 #' @export
 
 
-entry_rate_na <- function(dat, nr = 0.95, note = FALSE) {
-    if(note)cat("[NOTE]","process NAs & special value rate is more than 95%\n")
+entry_rate_na <- function(dat, nr = 0.98, note = FALSE) {
+    if(note)cat_line(paste("-- Processing NAs & special value rate is more than", nr), col = love_color("dark_green"))
     n_row <- nrow(dat)
     NAs_rate <- vapply(dat, function(x) {
         sum(is.na(x)) / n_row
@@ -311,8 +326,7 @@ entry_rate_na <- function(dat, nr = 0.95, note = FALSE) {
 #'
 #' @export
 low_variance_filter <- function(dat, lvp = 0.97, note = FALSE) {
-    if (note) {
-        (cat("[NOTE] delete low variance variables.\n"))
+    if (note) {cat_line(paste("-- Deleting low variance variables"), col = love_color("dark_green"))
     }
     dat <- dat[which(colSums(is.na(dat)) != nrow(dat))]
     is_na <- as.data.frame(abs(is.na(dat)))
