@@ -25,83 +25,95 @@
 #' dat_train = train_test$train
 #' dat_test = train_test$test
 #' @importFrom stats quantile ecdf
+#' @importFrom cli cat_rule cat_line cat_bullet
 #' @export
-
-train_test_split <- function(dat, prop = 0.7, split_type = "Random",
-occur_time = NULL, cut_date = NULL, start_date = NULL, save_data = FALSE,
-dir_path = tempdir(), file_name = NULL, note = FALSE, seed = 43) {
-
-    if (prop > 1 || !is.numeric(prop)) {
-        warning("[Invalid]  prop is not a numeric or more than 1,  reset to 0.7.\n")
-        prop = 0.7
-    }
-    if (!any(is.element(split_type, c("OOT", "Random", "byRow")))) {
-        stop("split_type must be either 'OOT' or 'Random' or 'byRow'.\n")
-    }
-    if (length(split_type) > 1) {
-        warning("your split_type is more than one and only the first one is selected.\n")
-    }
-    if (length(split_type) == 0) {
-        warning("split_type is missing,  set 'Random' by default.\n")
-        split_type = "Random"
-    }
-    if (split_type[1] == "OOT" & !is.null(occur_time) && any(names(dat) == occur_time)) {
-        dat = time_transfer(dat, date_cols = occur_time)
-        if (is_date(dat[, occur_time])) {
-            if (is.null(cut_date)) {
-                cut_date = date_cut(dat_time = dat[, occur_time], pct = prop)
-            }
-            if (is.null(start_date)) {
-                start_date = date_cut(dat_time = dat[, occur_time], pct = 0)
-            }
-            dat[, occur_time] = as.Date(dat[, occur_time])
-            test = dat[which(dat[, occur_time] >= cut_date),]
-            train = dat[which(dat[, occur_time] >= start_date & dat[, occur_time] < cut_date),]
-            if (note) cat(paste("[NOTE]", "total:", nrow(dat), "--->test:",
-            nrow(test), " train:", nrow(train), ".\n", sep = "", collapse = "\n"))
-            } else {
-                if (!is.null(seed)) set.seed(seed) else set.seed(46)
-                sub = sample(1:nrow(dat), round(nrow(dat) * prop))
-                train = dat[sub,]
-                test = dat[-sub,]
-                if (note) cat(paste("[NOTE]", "total:", nrow(dat), "--->test:",
-                nrow(test), " train:", nrow(train), ".\n", sep = "", collapse = "\n"))
-                warning(paste(occur_time, "is  not date or time, unable to use OOT , split random.\n"))
-            }
-
+train_test_split <- function( dat, prop = 0.7, split_type = "Random",occur_time = NULL,
+                              cut_date = NULL, start_date = NULL, save_data = FALSE, 
+                              dir_path = tempdir(), file_name = NULL, note = FALSE, seed = 43) {
+  
+  if (prop > 1 || !is.numeric(prop)) {
+    warning("-- prop is not a numeric or more than 1,  reset to 0.7.\n")
+    prop = 0.7
+  }
+  if (!any(is.element(split_type, c("OOT", "Random", "byRow")))) {
+    stop("split_type must be either 'OOT' or 'Random' or 'byRow'.\n")
+  }
+  if (length(split_type) > 1) {
+    warning("-- your split_type is more than one and only the first one is selected.\n")
+  }
+  if (length(split_type) == 0) {
+    warning("-- split_type is missing, set 'Random' by default.\n")
+    split_type = "Random"
+  }
+  if (split_type[1] == "OOT" & !is.null(occur_time) && any(names(dat) == occur_time)) {
+    dat = time_transfer(dat, date_cols = occur_time)
+    if (is_date(dat[, occur_time])) {
+      if (is.null(cut_date)) {
+        cut_date = date_cut(dat_time = dat[, occur_time], pct = prop)
+      }
+      if (is.null(start_date)) {
+        start_date = date_cut(dat_time = dat[, occur_time], pct = 0)
+      }
+      dat[, occur_time] = as.Date(dat[, occur_time])
+      test = dat[which(dat[, occur_time] >= cut_date),]
+      train = dat[which(dat[, occur_time] >= start_date & dat[, occur_time] < cut_date),]
+      if (note){				
+        cat_line("-- train_test_split:", col = love_color("dark_green"))
+        split_out = list(Total = nrow(dat), Train = nrow(train), Test = nrow(test))
+        cat_bullet(paste0(format(names(split_out)), ": ", unname(split_out),
+                          " (", sapply(split_out,function(x)as_percent(x/nrow(dat),2)),")"), col = "darkgrey") 
+      }
     } else {
-        if (!is.null(seed)) set.seed(seed) else set.seed(46)
-        sub = sample(1:nrow(dat), round(nrow(dat) * prop))
-        train = dat[sub,]
-        test = dat[-sub,]
-        if (note) cat(paste("[NOTE]", "total:", nrow(dat), "--->test:",
-            nrow(test), " train:", nrow(train), ".\n", sep = "", collapse = "\n"))
-        }
-    if (split_type[1] == "Random") {
-        if (!is.null(seed)) set.seed(seed) else set.seed(46)
-        sub = sample(1:nrow(dat), round(nrow(dat) * prop))
-        train = dat[sub,]
-        test = dat[-sub,]
-        if (note) cat(paste("[NOTE]", "total:", nrow(dat), "--->test:",
-        nrow(test), " train:", nrow(train), ".\n", sep = "", collapse = "\n"))
-        }
+      if (!is.null(seed)) set.seed(seed) else set.seed(46)
+      sub = sample(1:nrow(dat), round(nrow(dat) * prop))
+      train = dat[sub,]
+      test = dat[-sub,]
+      if (note){				
+        cat_line("-- train_test_split:", col = love_color("dark_green"))
+        split_out = list(Total = nrow(dat), Train = nrow(train), Test = nrow(test))
+        cat_bullet(paste0(format(names(split_out)), ": ", unname(split_out),
+                          " (", sapply(split_out,function(x)as_percent(x/nrow(dat),2)),")"), col = "darkgrey")       
+      }
+      warning(paste(occur_time, "is  not date or time, unable to use OOT , split random.\n"))
+    }
+    
+  }else{
     if (split_type[1] == "byRow") {
         sub = 1:round(nrow(dat) * prop)
         train = dat[sub,]
         test = dat[-sub,]
-        if (note) (paste("[NOTE]", "total:", nrow(dat), "--->test:",
-        nrow(test), " train:", nrow(train), ".\n", sep = "", collapse = "\n"))
+        if (note){				
+          cat_line("-- train_test_split:", col = love_color("dark_green"))
+          split_out = list(Total = nrow(dat), Train = nrow(train), Test = nrow(test))
+          cat_bullet(paste0(format(names(split_out)), ": ", unname(split_out),
+                            " (", sapply(split_out,function(x)as_percent(x/nrow(dat),2)),")"), col = "darkgrey")     
         }
-    if (save_data) {
-        dir_path = ifelse(!is.character(dir_path),
-                      tempdir(), dir_path)
-        if (!dir.exists(dir_path)) dir.create(dir_path)
-        if (!is.character(file_name)) file_name = NULL
-        save_dt(train, file_name = ifelse(is.null(file_name), "dat.train", paste(file_name, "dat.train", sep = ".")), dir_path = dir_path)
-        save_dt(test, file_name = ifelse(is.null(file_name), "dat.test", paste(file_name, "dat.test", sep = ".")), dir_path = dir_path)
+	 }else{
+	
+      if (!is.null(seed)) set.seed(seed) else set.seed(46)
+      sub = sample(1:nrow(dat), round(nrow(dat) * prop))
+      train = dat[sub,]
+      test = dat[-sub,]
+      if (note){				
+        cat_line("-- train_test_split:", col = love_color("dark_green"))
+        split_out = list(Total = nrow(dat), Train = nrow(train), Test = nrow(test))
+        cat_bullet(paste0(format(names(split_out)), ": ", unname(split_out),
+                          " (", sapply(split_out,function(x)as_percent(x/nrow(dat),2)),")"), col = "darkgrey")     
+      }
     }
-    return(list(test = test, train = train))
+  }
+  
+  if (save_data) {
+    dir_path = ifelse(!is.character(dir_path),
+                      tempdir(), dir_path)
+    if (!dir.exists(dir_path)) dir.create(dir_path)
+    if (!is.character(file_name)) file_name = NULL
+    save_dt(train, file_name = ifelse(is.null(file_name), "train", paste0(file_name, "_train")), dir_path = dir_path)
+    save_dt(test, file_name = ifelse(is.null(file_name), "test", paste0(file_name, "_test")), dir_path = dir_path)
+  }
+  return(list(test = test, train = train))
 }
+ 
 
 
 #' Stratified Folds
@@ -118,6 +130,7 @@ dir_path = tempdir(), file_name = NULL, note = FALSE, seed = 43) {
 #' dat = UCICreditCard[sub,]
 #' @importFrom stats quantile ecdf
 #' @export
+
 cv_split <- function(dat, k = 5, occur_time = NULL, seed = 46) {
     cv_list = list()
     dat = checking_data(dat = dat, occur_time = occur_time)
@@ -176,17 +189,19 @@ cv_split <- function(dat, k = 5, occur_time = NULL, seed = 46) {
 #' \dontrun{
 #' require_packages(c("dplyr","ggplot2"))
 #' }
+#' @importFrom cli cat_rule cat_line cat_bullet
 #' @export
-require_packages <- function(pkg) {
-    opt = options("warn" = -1) # suppress warnings
-    new_pkg <- pkg[!(pkg %in% installed.packages()[, "Package"])]
-    if (length(new_pkg) > 0) {
-        install.packages(new_pkg, dependencies = TRUE)
-        cat("[NOTE] Installs missing packages if needed")
-        cat(paste("[NOTE]", "packages", new_pkg, " are installed!"))
-    }
-    sapply(pkg, require, ch = TRUE)
-    options(opt) # reset warnings
+require_packages <- function(pkg){
+   opt = options("warn" = -1) # suppress warnings
+   new_pkg <- pkg[!(pkg %in% installed.packages()[, "Package"])]
+   if (length(new_pkg) > 0) {
+     install.packages(new_pkg, dependencies = TRUE)
+     cat_rule("Installing missing packages if needed", col = love_color("light_cyan"))
+     cat_line("-- Following packages  are installed and loaded:", col = love_color("dark_green"))
+     cat_bullet(paste0(format(new_pkg)), col = "darkgrey")   
+   }
+   sapply(pkg, require, ch = TRUE)
+   options(opt) # reset warnings
 }
 
 
@@ -201,7 +216,7 @@ require_packages <- function(pkg) {
 #' UCICreditCard = quick_as_df(UCICreditCard)
 #'
 #' @export
-quick_as_df <- function(df_list) {
+quick_as_df <- function(df_list){
     class(df_list) <- "data.frame"
     attr(df_list, "row.names") <- .set_row_names(length(df_list[[1]]))
     df_list
@@ -222,22 +237,151 @@ quick_as_df <- function(df_list) {
 #' save_dt(UCICreditCard,"UCICreditCard", tempdir())
 #' @importFrom data.table fwrite fread dcast melt
 #' @export
-
 save_dt <- function(dt, file_name = "dat", dir_path = getwd(),
-                    note = FALSE, as_list = FALSE, row_names = FALSE, append = FALSE) {
+                     note = FALSE, as_list = FALSE, row_names = FALSE, append = FALSE){
 
-    if (!dir.exists(dir_path)) dir.create(dir_path)
-    if (dir.exists(paste0(dir_path, '/', file_name, ".csv")))file.remove(list.files(paste0(dir_path, '/', file_name, ".csv"), recursive = TRUE, full.names = TRUE))
-    if (note) {
-        cat(paste0("[NOTE]", "Saved to ", "'",paste0(dir_path, '/', file_name, ".csv"),"'\n"))
+   if (!dir.exists(dir_path)) dir.create(dir_path)
+   if (dir.exists(paste0(dir_path, '/', file_name, ".csv")))file.remove(list.files(paste0(dir_path, '/', file_name, ".csv"), recursive = TRUE, full.names = TRUE))
+   if (note) {
+     cat_line(paste("-- Saving",file_name, "to:"), col = love_color("dark_green"))
+     cat_bullet(paste0(format(paste0(dir_path, '/', file_name, ".csv"))), col = "darkgrey")
+
+   }
+   if (as_list) {
+     fwrite(list(dt), paste0(dir_path, '/', file_name, ".csv"),
+            append = append,col.names = FALSE)
+   } else {
+     fwrite(as.data.frame(dt), paste0(dir_path, '/', file_name, ".csv"),
+            append = append, row.names = row_names)
+   }
+}
+
+
+#' Read data
+#'
+#' \code{read_dt} is for loading data.
+#' @param path Path to file or file name in working directory & path to file.
+#' @param encoding Default is "unknown". Other possible options are "UTF-8" and "Latin-1". 
+#' @param header  Does the first data line contain column names? 
+#' @param sep  The separator between columns.
+#' @param stringsAsFactors  Logical. Convert all character columns to factors?
+#' @importFrom data.table fwrite fread dcast melt
+#' @importFrom dplyr distinct
+#' @importFrom cli cat_rule cat_line cat_bullet
+#' @export
+
+
+read_dt = function(path, encoding = "unknown", header = 'auto', sep = "auto", stringsAsFactors = FALSE){
+  file_names = sort(list.files(path))
+  if (length(file_names) > 0) {
+    file_names = file_names[grepl("[.]", file_names)]
+    
+    file_format = c()
+    for (i in file_names) {
+      path_file = paste(path, i, sep = "/")
+      file_format[i] = check_data_format(path_file)
     }
-    if (as_list) {
-        fwrite(list(dt), paste0(dir_path, '/', file_name, ".csv"),
-               append = append,col.names = FALSE)
+    
+    file_format_tx = file_format[!is.na(file_format) && !grepl('xl\\S{1,2}$',file_format)]
+    
+    dt_list = list()
+    if(length(file_format_tx)>0){
+      for (file_name in names(file_format_tx)) {
+        file_n = gsub('.csv$|.txt$|.CSV$', "",file_name)
+        dt_list[[file_n]] = dplyr::distinct(quick_as_df(data.table::fread(paste(path, file_name, sep = "/"),
+                                                                          encoding = encoding, header = header,
+                                                                          sep = sep, stringsAsFactors = stringsAsFactors)))
+      }
+    }
+    cat_line("-- Input files:", col = love_color("dark_green"))
+    cat_bullet(paste0(format(names(file_format_tx))), col = "darkgrey")
+	return(dt_list)
+  } else {
+    
+    file_format = check_data_format(path)
+    file_format_tx = file_format[!is.na(file_format) && !grepl('xl\\S{1,2}$',file_format)]
+    #file_format_tc = file_format[!is.na(file_format)& file_format %alike% 'csv$|txt$|CSV$']
+    if (length(file_format_tx) > 0) {
+      quick_as_df(data.table::fread(paste(path, sep = "/"),
+                                    encoding = encoding, header = header,
+                                    sep = sep, stringsAsFactors = stringsAsFactors))
     } else {
-        fwrite(as.data.frame(dt), paste0(dir_path, '/', file_name, ".csv"),
-               append = append, row.names = row_names)
+      stop(paste0("Cannot open file '", path, "'"))
     }
+  }
+}
+
+#' @rdname read_dt
+check_data_format <- function(path) {
+  if (file.exists(path)) {
+    sig = readBin(path, n = 10, what = "raw")
+    xlsx_sig <- as.raw(c(
+      "0x50", "0x4B", "0x03", "0x04"
+    ))
+    xls_sig <- as.raw(c(
+      "0xD0", "0xCF", "0x11", "0xE0", "0xA1", "0xB1", "0x1A", "0xE1"
+    ))
+    data_format = NA
+    if (identical(sig[1:4], xlsx_sig)) {
+      data_format = "xlsx"
+    } else if (identical(sig[1:4], xls_sig)) {
+      data_format = "xls"
+    } else if (grepl(".txt$", path)) {
+      data_format = "txt"
+    } else if (grepl(".csv$|.CVS$", path)) {
+      data_format = "csv"
+    }
+
+  } else {
+    stop(paste0("Cannot open file '", path, "': no such file or directory"))
+  }
+  data_format
+}
+
+#' multi_left_jion
+#'
+#' \code{multi_left_jion} is for left jion a list of datasets fast.
+#' @param df_list A list of datasets.
+#' @param key_dt Name or index of Key table to left join.
+#' @param by  Name of Key columns to join.
+#' @examples
+#' save_dt(UCICreditCard,"UCICreditCard", tempdir())
+#' @export
+
+
+multi_left_jion = function(df_list,key_dt = NULL, by = NULL){
+  
+  if(is.null(by)){
+    stop("Key columns 'by' is missing.\n ")
+  }
+  if(!is.null(key_dt)){
+    key_id =df_list[key_dt]
+    df_list[key_dt] = NULL
+    df_list = append(key_id,df_list)
+  }
+  for(i in 1:length(df_list)){
+    by_id = c(df_list[[i]][[by]])
+    if(length(by_id)>0){
+      
+      df_list[[i]] = quick_as_df(df_list[[i]])
+      df_list[[i]][['id']] = as.character(by_id)
+      df_list[[i]][[by]] = NULL
+    }else{
+      warning(paste(paste0("No column called '",by,"'in"),names(df_list[i]),". Drop it...\n"))
+      df_list[i] = NULL
+    }
+  }
+
+  merge_func = function(x,y){
+    intersect_name = intersect(colnames(x),colnames(y))
+    intersect_name = intersect_name[-which(intersect_name == 'id')]
+    if(length(intersect_name)>0){
+      y = subset(y, select = -which(colnames(y) %in% intersect_name))
+    }
+    merge(x, y, by = 'id', all.x = TRUE)
+  }
+  dat_merge =  Reduce("merge_func",df_list) %>% re_name("id", by)
+  return(dat_merge)
 }
 
 #' Number of digits
@@ -253,8 +397,7 @@ save_dt <- function(dt, file_name = "dat", dir_path = getwd(),
 #' @export
 
 
-digits_num =function(dat_x) {
-    options(scipen = 200, stringsAsFactors = FALSE)
+digits_num =function(dat_x){
     digits1 = digits2 = 16
     dat_x = unique(unlist(dat_x[!is.na(dat_x)]))
     if (length(dat_x) > 0 && any(is.element(class(dat_x), c("integer", "numeric",
@@ -295,7 +438,7 @@ digits_num =function(dat_x) {
 #' @examples
 #' is_date(lendingclub$issue_d)
 #' @export
-is_date = function(x) {
+is_date = function(x){
     any(class(x) %in% c("Date", "POSIXlt", "POSIXct", "POSIXt"))
 }
 
@@ -311,7 +454,7 @@ is_date = function(x) {
 #' @importFrom stats quantile ecdf
 #' @export
 
-date_cut = function(dat_time, pct = 0.7) {
+date_cut = function(dat_time, pct = 0.7){
     dat_time = as.Date(dat_time)
     if (is_date(dat_time)) {
         date_n = quantile(ecdf(dat_time), seq(0, 1, by = 0.01))
@@ -358,7 +501,7 @@ as_percent <- function(x, digits = 2) {
 
 '%islike%' <- function(x, y) {
     grx = FALSE
-    x = gsub("\\$|\\*|\\+|\\?|\\[|\\^|\\{|\\}|\\\\|\\(|\\)|\\|\\)|\\]", "", x)
+    x = gsub("[^\u4e00-\u9fa5,^a-zA-Z,^0-9,^.,^_,^;^-]", "", x)
     y = gsub("\\{|\\}", "", y)
 
     if (any(x != '') & any(y != '') & any(!is.null(x)) & any(!is.null(y)) & any(!is.na(x)) & any(!is.na(y))) {
@@ -385,7 +528,7 @@ as_percent <- function(x, digits = 2) {
 #' @export
 
 '%alike%' <- function(x, y) {
-    x = gsub("\\$|\\*|\\+|\\?|\\[|\\^|\\{|\\}|\\\\|\\(|\\)|\\|\\)|\\]", "", x)
+    x = gsub("[^\u4e00-\u9fa5,^a-zA-Z,^0-9,^.,^_,^;^-]", "", x)
     if (any(x != '') & any(y != '') & any(!is.null(x)) & any(!is.null(y)) & any(!is.na(x)) & any(!is.na(y))) {
         y = unlist(y)
         y = y[which(y != '')]
@@ -418,8 +561,8 @@ as_percent <- function(x, digits = 2) {
 #' @export
 
 
-re_name = function(dat, oldname = c(), newname = c()) {
-    ind <- which(names(dat) %in% oldname)
+re_name <- function(dat, oldname = c(), newname = c()) {
+    ind = which(names(dat) %in% oldname)
     names(dat)[ind] = newname
     dat
 }
@@ -504,28 +647,28 @@ colSds <- function(x, na.rm = TRUE) {
 #' @rdname rowAny
 #' @export
 rowMaxs <- function(x, na.rm = FALSE) {
-    maxs <- apply(x, 1, function(i) ifelse(length(i) > 1, i[which.max(i)], i))
+    maxs = apply(x, 1, function(i) ifelse(length(i) > 1, i[which.max(i)], i))
     as.numeric(maxs)
 }
 
 #' @rdname rowAny
 #' @export
 rowMins <- function(x, na.rm = FALSE) {
-    mins <- apply(x, 1, function(i) i[which.min(i)])
+    mins = apply(x, 1, function(i) i[which.min(i)])
     as.numeric(mins)
 }
 
 #' @rdname rowAny
 #' @export
 rowMaxMins <- function(x, na.rm = FALSE) {
-    max_mins <- apply(x, 1, function(i) i[which.max(i)] - i[which.min(i)])
+    max_mins = apply(x, 1, function(i) i[which.max(i)] - i[which.min(i)])
     as.numeric(max_mins)
 }
 
 #' @rdname rowAny
 #' @export
 colMaxMins <- function(x, na.rm = FALSE) {
-    max_mins <- apply(x, 2, function(i) i[which.max(i)] - i[which.min(i)])
+    max_mins = apply(x, 2, function(i) i[which.max(i)] - i[which.min(i)])
     as.numeric(max_mins)
 }
 
@@ -547,34 +690,36 @@ colMaxMins <- function(x, na.rm = FALSE) {
 #' @export
 
 
-
-get_names <- function(dat, types = c('logical', 'factor', 'character', 'numeric',
-                                        'integer', 'double', "Date", "POSIXlt", "POSIXct", "POSIXt"), ex_cols = NULL, get_ex = FALSE) {
-    if (is.null(types)) {
-        stop("types is missing!")
-    }
-    if (is.null(ex_cols)) {
-        sel_names = names(dat)[sapply(dat, function(x) any(is.element(class(x), types)))]
-        ex_names = names(dat)[!(sapply(dat, function(x) any(is.element(class(x), types))))]
-    } else {
-        var_names = names(dat)[sapply(dat, function(x) any(is.element(class(x), types)))]
-        if (length(ex_cols) > 1 || !grepl("\\$|\\*|\\+|\\?|\\[|\\^|\\{|\\}|\\\\|\\|\\)|\\]", ex_cols) ) {
-            ex_vars = names(dat)[colnames(dat) %in% ex_cols]
-        } else {
-            ex_vars = names(dat)[colnames(dat) %alike% ex_cols]
-        }
-        ex_types = names(dat)[!(sapply(dat, function(x) any(is.element(class(x), types))))]
-        ex_names = unique(c(ex_vars, ex_types))
-        sel_names = setdiff(var_names, ex_names)
-    }
-    if (get_ex) {
-        dat = dat[ex_names]
-    } else {
-        dat = dat[sel_names]
-    }
-    var_names <- names(dat)
-    return(var_names)
+get_names <- function(dat, types = c('logical', 'factor', 'character', 'numeric','integer64',
+                                      'integer', 'double', "Date", "POSIXlt", "POSIXct", "POSIXt"),
+                       ex_cols = NULL, get_ex = FALSE) {
+   if (is.null(types)) {
+     stop("types is missing!")
+   }
+   if (is.null(ex_cols)) {
+     sel_names = names(dat)[sapply(dat, function(x) any(is.element(class(x), types)))]
+     ex_names = names(dat)[!(sapply(dat, function(x) any(is.element(class(x), types))))]
+   } else {
+     var_names = names(dat)[sapply(dat, function(x) any(is.element(class(x), types)))]
+     if (length(ex_cols) == 1 & !any(grepl("\\$|\\*|\\+|\\?|\\[|\\^|\\{|\\}|\\\\|\\|\\)|\\]", ex_cols)) ) {
+       ex_vars = names(dat)[colnames(dat) %in% ex_cols]
+     } else {
+       ex_vars = names(dat)[colnames(dat) %alike% ex_cols]
+     }
+     ex_types = names(dat)[!(sapply(dat, function(x) any(is.element(class(x), types))))]
+     ex_names = unique(c(ex_vars, ex_types))
+     sel_names = setdiff(var_names, ex_names)
+   }
+   if (get_ex) {
+     dat = dat[ex_names]
+   } else {
+     dat = dat[sel_names]
+   }
+   var_names <- names(dat)
+   return(var_names)
 }
+ 
+ 
 
 
 #' Get X List.
@@ -584,6 +729,7 @@ get_names <- function(dat, types = c('logical', 'factor', 'character', 'numeric'
 #' @param dat_test  Another data.frame.
 #' @param x_list Names of independent variables.
 #' @param ex_cols A list of excluded variables. Regular expressions can also be used to match variable names. Default is NULL.
+#' @param note Logical. Outputs info. Default is TRUE.
 #' @return  A list contains names of variables
 #' @seealso \code{\link{get_names}}
 #' @examples
@@ -592,7 +738,7 @@ get_names <- function(dat, types = c('logical', 'factor', 'character', 'numeric'
 #' @export
 
 
-get_x_list <- function(dat_train = NULL, dat_test = NULL,x_list = NULL, ex_cols = NULL) {
+get_x_list <- function(dat_train = NULL, dat_test = NULL,x_list = NULL, ex_cols = NULL,note = FALSE) {
     if (!is.null(dat_train)) {
         if (is.null(x_list) | length(x_list) <1 ) {
             if (is.null(dat_test)) {
@@ -623,17 +769,20 @@ get_x_list <- function(dat_train = NULL, dat_test = NULL,x_list = NULL, ex_cols
                                      ex_cols = ex_cols, get_ex = FALSE)
                 x_list_ts = intersect(x_list_t, x_list_s)
                 x_excluded_ts = setdiff(x_list_t, x_list_s)
-                if (length(x_excluded_ts) > 0) {
-                    cat(paste("following variables are not both in train & test :\n",
-                              paste(x_excluded_ts, collapse = " \n"), ".\n",sep = ""))
+                if (note & length(x_excluded_ts) > 0) {
+					cat_line("-- Following variables are not both in train & test:", col = love_color("dark_green"))
+                    cat_bullet(paste0(format(x_excluded_ts)), col = "darkgrey")
                 }
                 x_input = x_list %in% x_list_ts
             }
             if (!all(x_input)) {
                 x_retain = x_list[which(x_input)]
                 x_excluded = x_list[which(!x_input)]
-                cat(paste("following variables are excluded: \n",
-                          paste(x_excluded, collapse = " \n"), ".\n",sep = ""))
+				
+			if(note){
+			cat_line("-- Following variables are excluded:", col = love_color("dark_green"))
+            cat_bullet(paste0(format(x_excluded)), col = "darkgrey")
+            }
             } else {
                 x_retain = x_list
             }
@@ -643,8 +792,11 @@ get_x_list <- function(dat_train = NULL, dat_test = NULL,x_list = NULL, ex_cols
             if (!all(x_type)) {
                 x_list_retain = x_retain[which(x_type)]
                 x_excluded = x_retain[which(!x_type)]
-                cat(paste("following variables are not numeric or character or factor:\n",
-                          paste(x_excluded, collapse = " \n"), ".\n", sep = ""))
+				if(note){
+				cat_line("-- Following variables are not numeric or character or factor:", col = love_color("dark_green"))
+                cat_bullet(paste0(format(x_excluded)), col = "darkgrey")
+				}
+
             } else {
                 x_list_retain = x_retain
             }
@@ -669,61 +821,54 @@ get_x_list <- function(dat_train = NULL, dat_test = NULL,x_list = NULL, ex_cols
 #' @export
 
 
-
-
 checking_data <- function(dat = NULL, target = NULL, occur_time = NULL,
                           note = FALSE, pos_flag = NULL) {
-    if (note) {
-        (cat("[NOTE] checking dat and target format.\n"))
+  if (note)cat_line("-- Checking data and target format...", col = love_color("dark_green"))
+  if (is.null(dat)) {
+    warning("dat is null.\n")
+  } else {
+    if (!(class(dat)[1] == "data.frame")) {
+      if (any(is.element(class(dat), c("data.table", "list", "tbl_df", "tbl", "matrix"))) && length(dim(dat)) == 2) {
+        dat = as.data.frame(dat)
+        cat(paste("[NOTE]", "convert", class(dat)[1], "to data.frame.\n"))
+      } else {
+        warning("Dat is not two-dimensional.\n")
+      }
     }
-    if (is.null(dat)) {
-        warning("dat is null.\n")
+  }
+  if (!is.null(target)) {
+    if (!is.character(target) || length(target) > 1) {
+      warning(paste("target is not a string or a name.\n", sep = "\t"))
     } else {
-        if (!(class(dat)[1] == "data.frame")) {
-            if (any(is.element(class(dat), c("data.table", "list", "tbl_df", "tbl", "matrix"))) && length(dim(dat)) == 2) {
-                dat = as.data.frame(dat)
-                cat(paste("[NOTE]", "convert", class(dat)[1], "to data.frame.\n"))
-            } else {
-                warning("Dat is not two-dimensional.\n")
-            }
+      if (length(unique(dat[, target])) < 2) {
+        warning(paste("Unique values of target is only one.\n", sep = "\t"))
+      } else {
+        if (is.null(pos_flag)) {
+          pos_flag = list("1", "bad", 1, "Bad", "positive", "pos", "Positive", "Pos")
         }
-    }
-    if (!is.null(target)) {
-        if (!is.character(target) || length(target) > 1) {
-            warning(paste("target is not a string or a name.\n", sep = "\t"))
+        if (length(which(dat[, target] %in% pos_flag)) != 0) {
+          if (!all(sort(unique(dat[, target])) == c(0, 1))) {
+            pos = unique(dat[, target])[which(unique(dat[, target]) %in% pos_flag)]
+            dat[, target] = ifelse(dat[, target] %in% pos_flag, 1, 0)
+            warning(paste("The  values in of target has been encoded", pos, "= 1 and others = 0", " \n"))
+          }
         } else {
-            if (length(unique(dat[, target])) < 2) {
-                warning(paste("Unique values of target is only one.\n", sep = "\t"))
-            } else {
-                if (length(unique(dat[, target])) == 2) {
-
-                    if (is.null(pos_flag)) {
-                        pos_flag = list("1", "bad", 1, "Bad", "positive", "pos", "Positive", "Pos")
-                    }
-                    if (length(which(dat[, target] %in% pos_flag)) != 0) {
-                        if (!is.numeric(dat[, target]) || is.numeric(dat[, target]) && !all(sort(unique(dat[, target])) == c(0, 1))) {
-                            pos <- unique(dat[, target])[which(unique(dat[, target]) %in% pos_flag)]
-                            dat[, target] = ifelse(dat[, target] %in% pos_flag, 1, 0)
-                            warning(paste("The  values in of target has been encoded", pos, "=1 and others = 0", " \n"))
-                        }
-                    } else {
-                        warning(paste("Positive values of", target, "is not in pos_flag:", paste(pos_flag, collapse = ","), "\nplease set pos_flag. \n", sep = "\t"))
-                    }
-                }
-            }
+          warning(paste("Positive values of", target, "is not in pos_flag:", paste(pos_flag, collapse = ","), "\nplease set pos_flag. \n", sep = "\t"))
         }
+      }
     }
-    if (!is.null(occur_time)) {
-        if (is.element(occur_time, names(dat))) {
-            dat <- time_transfer(dat, date_cols = occur_time)
-            if (!is_date(dat[, occur_time])) {
-                warning(paste("occur_time:", occur_time, "is not time or date.\n", sep = "\t"))
-            }
-        } else {
-            warning(paste("occur_time:", occur_time, "is not in data.\n", sep = "\t"))
-        }
+  }
+  if (!is.null(occur_time)) {
+    if (is.element(occur_time, names(dat))) {
+      dat = time_transfer(dat, date_cols = occur_time)
+      if (!is_date(dat[, occur_time])) {
+        warning(paste("occur_time:", occur_time, "is not time or date.\n", sep = "\t"))
+      }
+    } else {
+      warning(paste("occur_time:", occur_time, "is not in data.\n", sep = "\t"))
     }
-    return(dat)
+  }
+  return(dat)
 }
 
 #' Parallel computing and export variables to global Env.
@@ -741,29 +886,29 @@ start_parallel_computing <- function(parallel = TRUE) {
     parallelType <- if (.Platform$OS.type == "windows")
         "snow" else "multicore"
 
-    numCores <- parallel::detectCores()
+    numCores = parallel::detectCores()
     attr(parallel, "type") <- parallelType
     attr(parallel, "cores") <- numCores
 
     if (parallel) {
         if (parallelType == "snow") {
 
-            cl <- parallel::makeCluster(numCores, type = "PSOCK")
+            cl = parallel::makeCluster(numCores, type = "PSOCK")
             attr(parallel, "cluster") <- cl
 
-            varlist <- ls(envir = parent.frame(), all.names = TRUE)
-            varlist <- varlist[varlist != "..."]
+            varlist = ls(envir = parent.frame(), all.names = TRUE)
+            varlist = varlist[varlist != "..."]
             parallel::clusterExport(cl, varlist = varlist, envir = parent.frame())
             parallel::clusterExport(cl, varlist = ls(envir = globalenv(), all.names = TRUE), envir = globalenv())
 
-            pkgs <- .packages()
+            pkgs = .packages()
             lapply(pkgs, function(pkg)
                 parallel::clusterCall(cl, library, package = pkg,
                                      character.only = TRUE))
             doParallel::registerDoParallel(cl, cores = numCores)
         }
         else if (parallelType == "multicore") {
-            cl <- parallel::makeCluster(numCores, type = "FORK")
+            cl = parallel::makeCluster(numCores, type = "FORK")
 
             doParallel::registerDoParallel(cl, cores = numCores[1])
 
@@ -819,7 +964,7 @@ loop_function <- function(func = NULL, args = list(data = NULL), x_list = NULL,
     opt = options(scipen = 200, stringsAsFactors = FALSE, "warn" = -1) # suppress warnings
     df_list = df_tbl = NULL
     if (parallel) {
-        parallel <- start_parallel_computing(parallel)
+        parallel = start_parallel_computing(parallel)
         stopCluster <- TRUE
     } else {
         parallel <- stopCluster <- FALSE
@@ -838,24 +983,57 @@ loop_function <- function(func = NULL, args = list(data = NULL), x_list = NULL,
         funct = function(i.) {
             try(do.call(func, c(args, x = i.)), silent = FALSE)
         }
-        df_list <- lapply(x_list, funct)
+        df_list = lapply(x_list, funct)
         if (as_list) {
-            df_tbl <- df_list
-            names(df_tbl) <- x_list
+            df_tbl = df_list
+            names(df_tbl) = x_list
         } else {
-            df_tbl <- as.data.frame(Reduce(bind, df_list))
+            df_tbl = as.data.frame(Reduce(bind, df_list))
         }
     } else {
-        df_list <- foreach(i. = x_list, .errorhandling = c('pass')) %dopar% {
+        df_list = foreach(i. = x_list, .errorhandling = c('pass')) %dopar% {
             try(do.call(func, args = c(args, x = i.)), silent = FALSE)
         }
         if (as_list) {
-            df_tbl <- df_list
-            names(df_tbl) <- x_list
+            df_tbl = df_list
+            names(df_tbl) = x_list
         } else {
-            df_tbl <- as.data.frame(Reduce(bind, df_list))
+            df_tbl = as.data.frame(Reduce(bind, df_list))
         }
     }
     options(opt) # reset warnings
     return(df_tbl)
 }
+
+
+
+#' string match
+#' #' \code{str_match} search for matches to argument pattern within each element of a character vector:
+#' @param pattern character string containing a regular expression (or character string for fixed = TRUE) to be matched in the given character vector. Coerced by as.character to a character string if possible. If a character vector of length 2 or more is supplied, the first element is used with a warning. Missing values are allowed except for regexpr and gregexpr.
+#' @param str_r	a character vector where matches are sought, or an object which can be coerced by as.character to a character vector. Long vectors are supported.
+#'
+#' @examples
+#' orignal_nam = c("12mdd","11mdd","10mdd")
+#' str_match(str_r = orignal_nam,pattern= "\\d+")
+#' @export
+
+str_match = function(pattern,str_r){
+  ind = regexpr(pattern,str_r )
+  substr(str_r, ind,attr(ind,"match.length"))
+}
+
+#' re_code
+#' #' \code{re_code} search for matches to argument pattern within each element of a character vector:
+#' @param x Variable to recode.
+#' @param codes	A data.frame of original value & recode value
+#' @examples
+#' SEX  = sample(c("F","M"),1000,replace = TRUE)
+#' codes= data.frame(ori_value = c('F','M'), code = c(0,1) )
+#' SEX_re = re_code(SEX,codes)
+#' @export
+re_code = function(x,codes){
+    for (i in 1:nrow(codes)) {
+    x[which(x == codes[i, 1])] = codes[i, 2]
+    }
+    return(x)
+}
\ No newline at end of file
