@@ -256,6 +256,75 @@ auc_value = function(target, prob) {
     u = sum(prob_1) - cnt_1 * (cnt_1 + 1) / 2
     exp(log(u) - log(cnt_1) - log(cnt_0))
 }
+#' ks_value
+#'
+#' \code{ks_value} is for get K-S value for a prob or score.
+#' @param prob A list of redict probability or score.
+#' @param target Vector of target.
+#' @return KS value
+#' @export
+
+ks_value <- function(target, prob) {
+  if(!is.numeric(target)){
+    target =  as.numeric(as.character(target))
+  }
+ sum_prob = as.data.frame(table(prob,target))
+ sum_prob <- data.table :: dcast(sum_prob, prob  ~ target, value.var = "Freq")
+ sum_prob[is.na(sum_prob)] = 0
+ sum_prob = data.frame(unclass(sum_prob))
+ cum_sum_1  = (cumsum(sum_prob$X1) / sum(sum_prob$X1))
+ cum_sum_0  = (cumsum(sum_prob$X0) / sum(sum_prob$X0))
+ KS = max(abs(round(cum_sum_1 - cum_sum_0, 4)), na.rm = TRUE)
+  return(KS)
+}
+
+
+#' tnr_value
+#'
+#' \code{tnr_value} is for get true negtive rate for a prob or score.
+#' @param prob A list of redict probability or score.
+#' @param target Vector of target.
+#' @return True Positive Rate
+#' @export
+
+tnr_value <- function(prob, target){
+   if(!is.numeric(target)){
+     target =  as.numeric(as.character(target))
+   }
+   tnr = ifelse(length(prob >= quantile(prob, 0.9,na.rm = TRUE)) > 0, 
+                mean(target[prob >= quantile(prob, 0.9,na.rm = TRUE)],na.rm = TRUE),0)
+   if(is.na(tnr)){tnr = 0}			
+   return(tnr)
+}
+ 
+#' lift_value
+#'
+#' \code{lift_value} is for get max lift value for a prob or score.
+#' @param prob A list of redict probability or score.
+#' @param target Vector of target.
+#' @return Max lift value
+#' @export
+
+lift_value <- function(target, prob) {
+   if(!is.numeric(target)){
+     target =  as.numeric(as.character(target))
+   }
+   t_prob = data.frame(prob,target)
+   t_prob = subset(t_prob, !is.na(prob))
+   breaks = cut_equal(prob,g = 10)
+   prob_bins = split_bins(dat =t_prob,x="prob",breaks)
+   sum_prob = as.data.frame(table(prob_bins,target = t_prob$target))
+   sum_prob = data.table :: dcast(sum_prob, prob_bins  ~ target, value.var = "Freq")
+   sum_prob[is.na(sum_prob)] = 0
+   sum_prob = data.frame(unclass(sum_prob))
+   sum_lift = sum_prob[order(sum_prob$prob, decreasing = TRUE),]
+   Lift = round((cumsum(sum_lift$X1) / ifelse(sum_lift$X0 + sum_lift$X1 > 0 ,cumsum(sum_lift$X0 + sum_lift$X1),1)) / 
+                  (sum(sum_lift$X1,na.rm = TRUE) / sum(sum_lift$X0 + sum_lift$X1, na.rm = TRUE)), 6)
+   MAX_Lift = mean(Lift, na.rm = TRUE)
+   return(MAX_Lift)
+}
+
+
 
 #' get central value.
 #'
